/*
 * Copyright 2017 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.jbpm.workbench.cm.client.perspectives;

import javax.enterprise.context.ApplicationScoped;

import org.jbpm.workbench.cm.client.overview.CaseOverviewPresenter;
import org.uberfire.client.annotations.Perspective;
import org.uberfire.client.annotations.WorkbenchPerspective;
import org.uberfire.client.workbench.panels.impl.StaticWorkbenchPanelPresenter;
import org.uberfire.lifecycle.OnStartup;
import org.uberfire.mvp.PlaceRequest;
import org.uberfire.mvp.impl.DefaultPlaceRequest;
import org.uberfire.workbench.model.PerspectiveDefinition;
import org.uberfire.workbench.model.impl.PartDefinitionImpl;
import org.uberfire.workbench.model.impl.PerspectiveDefinitionImpl;

@ApplicationScoped
@WorkbenchPerspective(identifier = CaseOverviewPerspective.PERSPECTIVE_ID)
public class CaseOverviewPerspective {

    public static final String PERSPECTIVE_ID = "CaseOverview";

    private final DefaultPlaceRequest overview = new DefaultPlaceRequest(CaseOverviewPresenter.SCREEN_ID);

    @Perspective
    public PerspectiveDefinition getPerspective() {
        final PerspectiveDefinition perspectiveDefinition = new PerspectiveDefinitionImpl(StaticWorkbenchPanelPresenter.class.getName());
        perspectiveDefinition.setName(PERSPECTIVE_ID);
        perspectiveDefinition.getRoot().addPart(new PartDefinitionImpl(overview));
        return perspectiveDefinition;
    }

    @OnStartup
    public void onStartup(final PlaceRequest place) {
        overview.getParameters().clear();
        for (final String param : place.getParameterNames()) {
            overview.addParameter(param,
                                  place.getParameter(param,
                                                     null));
        }
    }

    protected DefaultPlaceRequest getOverview() {
        return overview;
    }
}